/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Localize.cpp
 *
 *  Created on: 2011/01/14
 *      Author: HARK Development Team
 */

#include <Localize.hpp>
#include <BandUtils.hpp>
#include <Logger.hpp>

using namespace std;
using namespace boost;
using namespace NSDimensionPromise;

namespace NSHarkBinaural
{

Localize::DirectionProbabilityVectorPtr
Localize::GetDirection(HarmonicSoundPtr sound, const BandInformation& bi,
                       DirectionJudge& judge, IPD::ptr_t ipd, IID::ptr_t iid, MethodFlag::Kind mflag, bool sortByProbability)
{
    static Logger& logger = Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME);

    // 1500Hzを境にして、切り分ける
    // 1500Hzを越えてしまうと、両耳間に複数の波が入ってしまう。1つ以下の波に保証する必要がある。
    pair<BandUtils::BandHarmonicVectorPtr, BandUtils::BandHarmonicVectorPtr> divided =
        BandUtils::DivideBands(*sound->bands, bi.CreateBand(Frequency::FromHz(1500)));
    LogFine(logger, format("divided by 1500 hz, <%d, %d>") % divided.first->size() % divided.second->size());

    //DirectionJudge::AngleProbabilityVectorPtr directions =
    DirectionJudge::DirectionProbabilityVectorPtr directions =
        judge.Judge(divided, ipd, iid, mflag, sortByProbability);
    LogFine(logger, format("judge %d") % directions->size());

    return directions;
}

Localize::DirectionProbabilityVectorVectorPtr
Localize::GetDirections(HarmonicSoundVectorPtr sounds, const BandInformation& bi,
                        DirectionJudge& judge, IPD::ptr_t ipd, IID::ptr_t iid, MethodFlag::Kind mflag, bool sortByProbability)
{
    static Logger& logger = Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME);

    if (!sounds)
    {
        LogSevere(logger, "sounds is null");
        throw invalid_argument("sounds is null");
    }
    DirectionProbabilityVectorVectorPtr result(new DirectionProbabilityVectorVector());
    result->reserve(sounds->size());
    for (HarmonicSoundVector::iterator it = sounds->begin(); it != sounds->end(); ++it)
    {
        result->push_back(GetDirection(*it, bi, judge, ipd, iid, mflag, sortByProbability));
    }
    return result;
}

}
